/* GNOME clock & mailcheck applet
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 */

#include <panel-applet-gconf.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

#include "clockmail.h"
#include "ui_fileops.h"


void property_load(AppData *ad)
{
	GConfClient *client;
	gchar *key;
	GError *error = NULL;
	
	g_free(ad->mail_file);
	ad->mail_file = NULL;
	g_free(ad->theme_file);
	ad->theme_file = NULL;
	g_free(ad->newmail_exec_cmd);
	ad->newmail_exec_cmd = NULL;
	g_free(ad->reader_exec_cmd);
	ad->reader_exec_cmd = NULL;

	client = gconf_client_get_default();
	key = panel_applet_get_preferences_key(ad->applet);

	if (gconf_client_dir_exists(client, key, NULL))
		{
		ad->am_pm_enable = panel_applet_gconf_get_bool(ad->applet, "12hour", &error);
		if (error)
			{
			g_print ("%s \n", error->message);
			g_error_free (error);
			error = NULL;
			}
		ad->always_blink =  panel_applet_gconf_get_bool(ad->applet, "blink", NULL);
		ad->mail_file =  panel_applet_gconf_get_string(ad->applet, "mailfile", NULL);
		ad->reader_exec_cmd =  panel_applet_gconf_get_string(ad->applet, "reader-command", NULL);
		ad->newmail_exec_cmd =  panel_applet_gconf_get_string(ad->applet, "newmail-command", NULL);
		ad->exec_cmd_on_newmail =  panel_applet_gconf_get_bool(ad->applet, "newmail-command-enable", NULL);
		ad->theme_file =  panel_applet_gconf_get_string(ad->applet, "theme", NULL);
		ad->use_gmt =  panel_applet_gconf_get_bool(ad->applet, "gmt-enable", NULL);
		ad->gmt_offset =  panel_applet_gconf_get_int(ad->applet, "gmt-offset", NULL);
		ad->mail_max =  panel_applet_gconf_get_int(ad->applet, "mailmax", NULL);
		}

	g_object_unref(G_OBJECT(client));
	g_free(key);
}

void property_save(void)
{
	/* no op */
}

static void newmail_exec_cb(GtkWidget *w, gpointer data)
{
	AppData *ad = data;
	ad->exec_cmd_on_newmail = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	panel_applet_gconf_set_bool(ad->applet, "newmail-command-enable", ad->exec_cmd_on_newmail, NULL);
}

static void am_pm_time_cb(GtkWidget *w, gpointer data)
{
	AppData *ad = data;
	ad->am_pm_enable = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	panel_applet_gconf_set_bool(ad->applet, "12hour", ad->am_pm_enable, NULL);
}

static void use_gmt_cb(GtkWidget *w, gpointer data)
{
	AppData *ad = data;
	ad->use_gmt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	panel_applet_gconf_set_bool(ad->applet, "gmt-enable", ad->use_gmt, NULL);
}

static void gmt_offset_cb(GtkObject *adj, gpointer data)
{
	AppData *ad = data;
	ad->gmt_offset = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(adj));
	panel_applet_gconf_set_int(ad->applet, "gmt-offset", ad->gmt_offset, NULL);
} 

static void always_blink_cb(GtkWidget *w, gpointer data)
{
	AppData *ad = data;
	ad->always_blink = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	panel_applet_gconf_set_bool(ad->applet, "blink", ad->always_blink, NULL);
}

static void mail_max_cb(GtkObject *adj, gpointer data)
{
	AppData *ad = data;
	ad->mail_max = (gint)gtk_adjustment_get_value(GTK_ADJUSTMENT(adj));
	panel_applet_gconf_set_int(ad->applet, "mailmax", ad->mail_max, NULL);
}

static void theme_selected_cb(GtkWidget *clist, gint row, gint column,
			      GdkEventButton *bevent, gpointer data)
{
	AppData *ad = data;
	const gchar *text;
       
	text = gtk_clist_get_row_data(GTK_CLIST(clist), row);
	if (text) gtk_entry_set_text(GTK_ENTRY(ad->theme_entry), text);

	g_free(ad->theme_file);
	ad->theme_file = g_strdup(text);
	panel_applet_gconf_set_string(ad->applet, "theme", ad->theme_file, NULL);

	reload_skin(ad);
}

static void theme_enter_cb(GtkWidget *entry, gpointer data)
{
	AppData *ad = data;
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(entry));
	if (!text || strlen(text) < 1) return;

	g_free(ad->theme_file);
	ad->theme_file = g_strdup(text);

	panel_applet_gconf_set_string(PANEL_APPLET(ad->applet), "theme", ad->theme_file, NULL);

	reload_skin(ad);
}

static void mailfile_enter_cb(GtkWidget *entry, gpointer data)
{
	AppData *ad = data;
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(entry));
	g_free(ad->mail_file);
	ad->mail_file = g_strdup(text);

	panel_applet_gconf_set_string(PANEL_APPLET(ad->applet), "mailfile", ad->mail_file, NULL);

	check_mail_file_status (TRUE, ad);
}

static void reader_enter_cb(GtkWidget *entry, gpointer data)
{
	AppData *ad = data;
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(entry));
	g_free(ad->reader_exec_cmd);
	ad->reader_exec_cmd = g_strdup(text);

	panel_applet_gconf_set_string(PANEL_APPLET(ad->applet), "reader-command", ad->reader_exec_cmd, NULL);
}

static void newmail_enter_cb(GtkWidget *entry, gpointer data)
{
	AppData *ad = data;
	const gchar *text;

	text = gtk_entry_get_text(GTK_ENTRY(entry));
	g_free(ad->newmail_exec_cmd);
	ad->newmail_exec_cmd = g_strdup(text);

	panel_applet_gconf_set_string(PANEL_APPLET(ad->applet), "newmail-command", ad->newmail_exec_cmd, NULL);
}

static gint sort_theme_list_cb(void *a, void *b)
{
	return strcmp((gchar *)a, (gchar *)b);
}

static void populate_theme_list(GtkWidget *clist)
{
	gint row;
	gchar *buf[2];
	GList *theme_list = NULL;
	GList *work;

	/* add default theme */
	buf[0] = _("None (default)");
	buf[1] = NULL;
	row = gtk_clist_append(GTK_CLIST(clist), buf);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, "");

	if (!path_list(CLOCKMAIL_THEME_PATH, NULL, &theme_list)) return;

	theme_list = g_list_sort(theme_list, (GCompareFunc) sort_theme_list_cb);

	work = theme_list;
	while (work)
		{
		gchar *themedata_file = g_strconcat(work->data, "/clockmaildata", NULL);

		if (g_file_exists(themedata_file))
			{
			buf[0] = (gchar *)filename_from_path(work->data);
			buf[1] = NULL;
			row = gtk_clist_append(GTK_CLIST(clist), buf);
			gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
						    work->data, (GtkDestroyNotify) g_free);
			work->data = NULL;
			}
		g_free(themedata_file);
		work = work->next;
		}

	path_list_free(theme_list);
}

static gint property_destroy_cb(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;

	ad->propwindow = NULL;
	ad->theme_entry = NULL;
	return FALSE;
}

static void property_response_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(widget);
}

void property_show(PanelApplet *applet, gpointer data)
{
	AppData *ad = data;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *vbox1;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *button;
	GtkObject *adj;
	GtkWidget *spin;
	GtkWidget *scrolled;
	GtkWidget *theme_clist;
	GtkWidget *entry;
	GtkWidget *notebook;
	gchar *theme_title[] = { N_("Themes:"), };

	if(ad->propwindow)
		{
                gdk_window_raise(ad->propwindow->window);
                return;
		}

	ad->propwindow = gtk_dialog_new_with_buttons(_("Clockmail Preferences"), NULL,
						     GTK_DIALOG_DESTROY_WITH_PARENT,
						     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
						     NULL);

	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ad->propwindow)->vbox), notebook, TRUE, TRUE, 0);
	gtk_widget_show(notebook);

	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);

	frame = gtk_frame_new(_("Clock"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	gtk_widget_show(vbox1);

	button = gtk_check_button_new_with_label (_("Display time in 12 hour format (AM/PM)"));
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), ad->am_pm_enable);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   GTK_SIGNAL_FUNC(am_pm_time_cb), ad);
	gtk_widget_show(button);

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start( GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_check_button_new_with_label (_("Display time relative to GMT (Greenwich Mean Time):"));
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), ad->use_gmt);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   GTK_SIGNAL_FUNC(use_gmt_cb), ad);
	gtk_widget_show(button);

	adj = gtk_adjustment_new((float)ad->gmt_offset, -12.0, 12.0, 1, 1, 1 );
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0 );
	gtk_box_pack_start( GTK_BOX(hbox), spin, FALSE, FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
			   GTK_SIGNAL_FUNC(gmt_offset_cb), ad);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
	gtk_widget_show(spin);

	frame = gtk_frame_new(_("Mail"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox1 = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width(GTK_CONTAINER (vbox1), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	gtk_widget_show(vbox1);

	/* mail file entry */
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Mail file:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	entry = gtk_entry_new_with_max_length(255);
	if (ad->mail_file) gtk_entry_set_text(GTK_ENTRY(entry), ad->mail_file);
	gtk_signal_connect(GTK_OBJECT(entry), "changed",
			   GTK_SIGNAL_FUNC(mailfile_enter_cb), ad);
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);

	/* newmail exec command */
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_check_button_new_with_label (_("When new mail is received run:"));
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), ad->exec_cmd_on_newmail);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   GTK_SIGNAL_FUNC(newmail_exec_cb), ad);
	gtk_widget_show(button);

	entry = gtk_entry_new_with_max_length(255);
	if (ad->newmail_exec_cmd) gtk_entry_set_text(GTK_ENTRY(entry), ad->newmail_exec_cmd);
	gtk_signal_connect(GTK_OBJECT(entry), "changed",
			   GTK_SIGNAL_FUNC(newmail_enter_cb), ad);
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);

	button = gtk_check_button_new_with_label (_("Always blink when any mail is waiting."));
	gtk_box_pack_start(GTK_BOX(vbox1), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), ad->always_blink);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			   GTK_SIGNAL_FUNC(always_blink_cb), ad);
	gtk_widget_show(button);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start( GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Number of messages to consider mailbox full:"));
	gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	adj = gtk_adjustment_new((float)ad->mail_max, 10.0, 9000.0, 1, 5, 50);
	spin = gtk_spin_button_new( GTK_ADJUSTMENT(adj), 1, 0 );
	gtk_widget_set_usize(spin, 100, -1);
	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
			   GTK_SIGNAL_FUNC(mail_max_cb), ad);
	gtk_signal_connect(GTK_OBJECT(spin), "changed",
			   GTK_SIGNAL_FUNC(mail_max_cb), ad);
	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
	gtk_widget_show(spin);

	/* reader exec command */
	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label= gtk_label_new (_("When clicked, run:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	entry = gtk_entry_new_with_max_length(255);
	if (ad->reader_exec_cmd) gtk_entry_set_text(GTK_ENTRY(entry), ad->reader_exec_cmd);
	gtk_signal_connect(GTK_OBJECT(entry), "changed",
				GTK_SIGNAL_FUNC(reader_enter_cb), ad);
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);

        label = gtk_label_new(_("General"));
        gtk_widget_show(vbox);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	/* theme tab */

	frame = gtk_frame_new(NULL);
	gtk_container_set_border_width (GTK_CONTAINER (frame), GNOME_PAD_SMALL);

	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show(vbox);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Theme file (directory):"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	entry = gtk_entry_new_with_max_length(255);
	if (ad->theme_file) gtk_entry_set_text(GTK_ENTRY(entry), ad->theme_file);
	gtk_signal_connect(GTK_OBJECT(entry), "activate",
			   GTK_SIGNAL_FUNC(theme_enter_cb), ad);
	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);
	ad->theme_entry = entry;

	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);

	/* theme list */
	theme_clist = gtk_clist_new_with_titles (1, theme_title);
	gtk_clist_column_titles_passive(GTK_CLIST(theme_clist)); 
	gtk_signal_connect(GTK_OBJECT(theme_clist), "select_row",
			   GTK_SIGNAL_FUNC(theme_selected_cb), ad);
	gtk_container_add(GTK_CONTAINER(scrolled), theme_clist);
	gtk_widget_show(theme_clist);

	populate_theme_list(theme_clist);

        label = gtk_label_new(_("Theme"));
        gtk_widget_show(frame);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);

	g_signal_connect(G_OBJECT(ad->propwindow),"response",
			 G_CALLBACK(property_response_cb), ad);
	gtk_signal_connect(GTK_OBJECT(ad->propwindow), "destroy",
			   GTK_SIGNAL_FUNC(property_destroy_cb), ad);

	gtk_widget_show_all(ad->propwindow);
} 
