/* GNOME clock & mailcheck applet
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 */

#include "clockmail.h"
#include "skin.h"
#include "item.h"
#include "button.h"
#include "number.h"
#include "clock.h"
#include "theme.h"
#include "def_theme.h"
#include "pixbuf_ops.h"
#include "update.h"

SkinData *skin_new(void)
{
	SkinData *s;
	s = g_new0(SkinData, 1);
	return s;
}

void skin_free(SkinData *s)
{
	if (!s) return;

	if (s->pixbuf) gdk_pixbuf_unref(s->pixbuf);
	if (s->overlay) gdk_pixbuf_unref(s->overlay);

	number_free(s->hour);
	number_free(s->min);
	number_free(s->sec);
	number_free(s->month);
	number_free(s->day);
	number_free(s->year);
	number_free(s->year2);
	number_free(s->mail_count);
	number_free(s->messages);

	digit_free(s->dig_small);
	digit_free(s->dig_large);

	item_free(s->mail);
	item_free(s->month_txt);
	item_free(s->week_txt);
	item_free(s->mail_amount);
	item_free(s->sec_anim);

	item_free(s->button_pix);
	button_free(s->button);

	clock_free(s->clock);

	g_free(s);
}

void skin_flush(AppData *ad)
{
	GdkPixmap *pixmap;

	if (!ad->display->window) return;

	/* FIXME: memory/time consuming, should keep a pixmap around for this */

	gdk_pixbuf_render_pixmap_and_mask(ad->skin->pixbuf, &pixmap, NULL, 128);

	gdk_window_set_back_pixmap(ad->display->window, pixmap, FALSE);
	gdk_window_clear(ad->display->window);

	gdk_pixmap_unref(pixmap);
}

void applet_skin_redraw_all(AppData *ad)
{
	update_time_and_date(ad, TRUE);
	update_mail_status(ad, TRUE);
}

void applet_skin_backing_sync(AppData *ad)
{
	PanelAppletBackgroundType type;
	GdkColor color;
	GdkPixmap *pixmap = NULL;

	if (!ad->skin) return;

	type = panel_applet_get_background(ad->applet, &color, &pixmap);

	if (type == PANEL_COLOR_BACKGROUND || type == PANEL_NO_BACKGROUND)
		{
		gint r, g, b;

		if (type == PANEL_COLOR_BACKGROUND)
			{
			r = color.red >> 8;
			g = color.green >> 8;
			b = color.blue >> 8;

			gtk_widget_modify_bg(GTK_WIDGET(ad->applet), GTK_STATE_NORMAL, &color);
			}
		else
			{
			GtkStyle *style;
			GtkRcStyle *rc_style;
			GdkColor *bg;

			rc_style = gtk_rc_style_new();
			gtk_widget_modify_style(GTK_WIDGET(ad->applet), rc_style);
			gtk_rc_style_unref(rc_style);

			style = gtk_widget_get_style(GTK_WIDGET(ad->applet));
			bg = &style->bg[GTK_STATE_NORMAL];
			r = bg->red >> 8;
			g = bg->green >> 8;
			b = bg->blue >> 8;
			}

		pixbuf_draw_rect_fill(ad->skin->pixbuf,
				      0, 0, ad->skin->width, ad->skin->height,
				      r, g, b, 255);
		}
	else
		{
		GdkColormap *cmap;

		cmap = gdk_window_get_colormap(GTK_WIDGET(ad->applet)->window);
		gdk_pixbuf_get_from_drawable(ad->skin->pixbuf, pixmap, cmap,
					     0, 0, 0, 0,
					     ad->skin->width, ad->skin->height);
		}

	if (pixmap) gdk_pixmap_unref(pixmap);
	
	pixbuf_copy_area_alpha(ad->skin->overlay, 0, 0,
			       ad->skin->pixbuf, 0, 0, ad->skin->width, ad->skin->height, 255);

	item_back_set(ad->skin->mail, ad->skin->pixbuf);
	item_back_set(ad->skin->month_txt, ad->skin->pixbuf);
	item_back_set(ad->skin->week_txt, ad->skin->pixbuf);
	item_back_set(ad->skin->mail_amount, ad->skin->pixbuf);
	item_back_set(ad->skin->sec_anim, ad->skin->pixbuf);
	item_back_set(ad->skin->button_pix, ad->skin->pixbuf);

	number_back_set(ad->skin->hour, ad->skin->pixbuf);
	number_back_set(ad->skin->min, ad->skin->pixbuf);
	number_back_set(ad->skin->sec, ad->skin->pixbuf);
	number_back_set(ad->skin->month, ad->skin->pixbuf);
	number_back_set(ad->skin->day, ad->skin->pixbuf);
	number_back_set(ad->skin->year, ad->skin->pixbuf);
	number_back_set(ad->skin->year2, ad->skin->pixbuf);
	number_back_set(ad->skin->mail_count, ad->skin->pixbuf);
	number_back_set(ad->skin->messages, ad->skin->pixbuf);

	clock_back_set(ad->skin->clock, ad->skin->pixbuf);

	button_draw(ad->skin->button, FALSE, FALSE, TRUE, ad->skin->pixbuf, FALSE);
	applet_skin_redraw_all(ad);

	skin_flush(ad);
}

void applet_skin_window_sync(AppData *ad)
{
	if (!ad->skin) return;

	gtk_widget_set_size_request(ad->display, ad->skin->width, ad->skin->height);
	gdk_window_resize(ad->display->window, ad->skin->width, ad->skin->height);
#if 0
	gtk_widget_set_usize(GTK_WIDGET(ad->applet), ad->skin->width, ad->skin->height);
#endif

	if (ad->skin->mail)
		ad->mail_sections = ad->skin->mail->sections;
	else
		ad->mail_sections = 1;

	applet_skin_backing_sync(ad);
}

static SkinData *skin_load_best_match(gchar *path, gint vertical, gint size, AppData *ad)
{
	SkinData *s = NULL;

	s = skin_load(path, vertical, size, ad);
	if (s) return s;

	/* tiny falls back to small */
	if (size == SIZEHINT_TINY)
		{
		s = skin_load(path, vertical, SIZEHINT_SMALL, ad);
		if (s) return s;
		}

	/* small falls back to tiny */
	if (size == SIZEHINT_SMALL)
		{
		s = skin_load(path, vertical, SIZEHINT_TINY, ad);
		if (s) return s;
		}

	/* huge falls back to large */
	if (size == SIZEHINT_HUGE)
		{
		s = skin_load(path, vertical, SIZEHINT_LARGE, ad);
		if (s) return s;
		}

	if (size == SIZEHINT_STANDARD) return NULL;

	/* failed so far, try standard */
	return skin_load(path, vertical, SIZEHINT_STANDARD, ad);
}

gint skin_set(gchar *path, AppData *ad)
{
	SkinData *new_s = NULL;
	SkinData *old_s = ad->skin;

	if (!path)
		{
		new_s = skin_load_default(ad);
		}
	else
		{
		new_s = skin_load_best_match(path,
					     (ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT),
					     ad->sizehint, ad);
		if (!new_s)
			{
			new_s = skin_load(path,
					  !(ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT),
					  SIZEHINT_STANDARD, ad);
			}
		}

	if (!new_s) return FALSE;

	ad->skin = new_s;

	applet_skin_window_sync(ad);
	ad->active = NULL;

	skin_free(old_s);

	return TRUE;
}


